package jdk.internal.org.objectweb.asm.commons;

import jdk.internal.org.objectweb.asm.Label;
import jdk.internal.org.objectweb.asm.Handle;
import jdk.internal.org.objectweb.asm.TypePath;
import jdk.internal.org.objectweb.asm.AnnotationVisitor;
import jdk.internal.org.objectweb.asm.MethodVisitor;

public class RemappingMethodAdapter extends LocalVariablesSorter
{
    protected final Remapper remapper;
    
    public RemappingMethodAdapter(int n, String s, MethodVisitor methodVisitor, Remapper remapper) {
        this(327680, n, s, methodVisitor, remapper);
    }
    
    protected RemappingMethodAdapter(int n, int n2, String s, MethodVisitor methodVisitor, Remapper remapper) {
        super(n, n2, s, methodVisitor);
        this.remapper = remapper;
    }
    
    @Override
    public AnnotationVisitor visitAnnotationDefault() {
        AnnotationVisitor visitAnnotationDefault = super.visitAnnotationDefault();
        return (visitAnnotationDefault == null) ? visitAnnotationDefault : new RemappingAnnotationAdapter(visitAnnotationDefault, this.remapper);
    }
    
    @Override
    public AnnotationVisitor visitAnnotation(String s, boolean visible) {
        AnnotationVisitor visitAnnotation = super.visitAnnotation(this.remapper.mapDesc(s), visible);
        return (visitAnnotation == null) ? visitAnnotation : new RemappingAnnotationAdapter(visitAnnotation, this.remapper);
    }
    
    @Override
    public AnnotationVisitor visitTypeAnnotation(int typeRef, TypePath typePath, String s, boolean visible) {
        AnnotationVisitor visitTypeAnnotation = super.visitTypeAnnotation(typeRef, typePath, this.remapper.mapDesc(s), visible);
        return (visitTypeAnnotation == null) ? visitTypeAnnotation : new RemappingAnnotationAdapter(visitTypeAnnotation, this.remapper);
    }
    
    @Override
    public AnnotationVisitor visitParameterAnnotation(int parameter, String s, boolean visible) {
        AnnotationVisitor visitParameterAnnotation = super.visitParameterAnnotation(parameter, this.remapper.mapDesc(s), visible);
        return (visitParameterAnnotation == null) ? visitParameterAnnotation : new RemappingAnnotationAdapter(visitParameterAnnotation, this.remapper);
    }
    
    @Override
    public void visitFrame(int n, int n2, Object[] array, int n3, Object[] array2) {
        super.visitFrame(n, n2, this.remapEntries(n2, array), n3, this.remapEntries(n3, array2));
    }
    
    private Object[] remapEntries(int n, Object[] array) {
        int i = 0;
        while (i < n) {
            if ((array[i] instanceof String)) {
                Object[] array2 = new Object[n];
                if (i > 0) {
                    System.arraycopy(array, 0, array2, 0, i);
                }
                do {
                    Object o = array[i];
                    array2[i++] = ((o instanceof String) ? this.remapper.mapType((String)o) : o);
                } while (i < n);
                return array2;
            }
            else {
                ++i;
            }
        }
        return array;
    }
    
    @Override
    public void visitFieldInsn(int opcode, String s, String s2, String s3) {
        super.visitFieldInsn(opcode, this.remapper.mapType(s), this.remapper.mapFieldName(s, s2, s3), this.remapper.mapDesc(s3));
    }
    
    @Deprecated
    @Override
    public void visitMethodInsn(int opcode, String owner, String name, String descriptor) {
        if (this.api >= 327680) {
            super.visitMethodInsn(opcode, owner, name, descriptor);
            return;
        }
        this.doVisitMethodInsn(opcode, owner, name, descriptor, opcode == 185);
        return;
    }
    
    @Override
    public void visitMethodInsn(int opcode, String owner, String name, String descriptor, boolean isInterface) {
        if (this.api < 327680) {
            super.visitMethodInsn(opcode, owner, name, descriptor, isInterface);
            return;
        }
        this.doVisitMethodInsn(opcode, owner, name, descriptor, isInterface);
        return;
    }
    
    private void doVisitMethodInsn(int opcode, String s, String s2, String s3, boolean isInterface) {
        if (this.mv != null) {
            this.mv.visitMethodInsn(opcode, this.remapper.mapType(s), this.remapper.mapMethodName(s, s2, s3), this.remapper.mapMethodDesc(s3), isInterface);
        }
    }
    
    @Override
    public void visitInvokeDynamicInsn(String s, String s2, Handle handle, Object... bootstrapMethodArguments) {
        for (int i = 0; i < bootstrapMethodArguments.length; ++i) {
            bootstrapMethodArguments[i] = this.remapper.mapValue(bootstrapMethodArguments[i]);
        }
        super.visitInvokeDynamicInsn(this.remapper.mapInvokeDynamicMethodName(s, s2), this.remapper.mapMethodDesc(s2), (Handle)this.remapper.mapValue(handle), bootstrapMethodArguments);
    }
    
    @Override
    public void visitTypeInsn(int opcode, String s) {
        super.visitTypeInsn(opcode, this.remapper.mapType(s));
    }
    
    @Override
    public void visitLdcInsn(Object o) {
        super.visitLdcInsn(this.remapper.mapValue(o));
    }
    
    @Override
    public void visitMultiANewArrayInsn(String s, int numDimensions) {
        super.visitMultiANewArrayInsn(this.remapper.mapDesc(s), numDimensions);
    }
    
    @Override
    public AnnotationVisitor visitInsnAnnotation(int typeRef, TypePath typePath, String s, boolean visible) {
        AnnotationVisitor visitInsnAnnotation = super.visitInsnAnnotation(typeRef, typePath, this.remapper.mapDesc(s), visible);
        return (visitInsnAnnotation == null) ? visitInsnAnnotation : new RemappingAnnotationAdapter(visitInsnAnnotation, this.remapper);
    }
    
    @Override
    public void visitTryCatchBlock(Label start, Label end, Label handler, String s) {
        super.visitTryCatchBlock(start, end, handler, (s == null) ? null : this.remapper.mapType(s));
    }
    
    @Override
    public AnnotationVisitor visitTryCatchAnnotation(int typeRef, TypePath typePath, String s, boolean visible) {
        AnnotationVisitor visitTryCatchAnnotation = super.visitTryCatchAnnotation(typeRef, typePath, this.remapper.mapDesc(s), visible);
        return (visitTryCatchAnnotation == null) ? visitTryCatchAnnotation : new RemappingAnnotationAdapter(visitTryCatchAnnotation, this.remapper);
    }
    
    @Override
    public void visitLocalVariable(String s, String s2, String s3, Label label, Label label2, int n) {
        super.visitLocalVariable(s, this.remapper.mapDesc(s2), this.remapper.mapSignature(s3, true), label, label2, n);
    }
    
    @Override
    public AnnotationVisitor visitLocalVariableAnnotation(int n, TypePath typePath, Label[] array, Label[] array2, int[] array3, String s, boolean b) {
        AnnotationVisitor visitLocalVariableAnnotation = super.visitLocalVariableAnnotation(n, typePath, array, array2, array3, this.remapper.mapDesc(s), b);
        return (visitLocalVariableAnnotation == null) ? visitLocalVariableAnnotation : new RemappingAnnotationAdapter(visitLocalVariableAnnotation, this.remapper);
    }
}
